import getpass
# import os

from libcloud.compute.providers import get_driver
from libcloud.compute.types import Provider

# reqs:
#   services: EC2 (nova, glance, neutron)
#   resources: 2 instances, 2 elastic ips (1 keypair, 2 security groups)

# The image to look for and use for the started instance
ubuntu_image_name = 'ubuntu/images/hvm-ssd/ubuntu-bionic-18.04-amd64-server-20200408'
# ubuntu_image_id = "ami-0e342d72b12109f91" # local ami id for resent ubuntu 18.04 20200408 in eu-central-1

# The public key to be used for SSH connection, please make sure, that you have the corresponding private key
#
# id_rsa.pub should look like this (standard sshd pubkey format):
# ssh-rsa AAAAB3NzaC1yc2EAAAABIwAAAQEAw+J...F3w2mleybgT1w== user@HOSTNAME

keypair_name = 'srieger-pub'
pub_key_file = '~/.ssh/id_rsa.pub'

flavor_name = 't2.nano'

# default region
# region_name = 'eu-central-1'
# region_name = 'ap-south-1'

# AWS Educate only allows us-east-1 see our AWS classroom at https://www.awseducate.com
# e.g., https://www.awseducate.com/student/s/launch-classroom?classroomId=a1v3m000005mNm6AAE

region_name = 'us-east-1'


def main():
    ###########################################################################
    #
    # get credentials
    #
    ###########################################################################

    # see AWS Educate classroom, Account Details

    # access_id = getpass.win_getpass("Enter your access_id:")
    # secret_key = getpass.win_getpass("Enter your secret_key:")
    # session_token = getpass.win_getpass("Enter your session_token:")
    access_id = "ASIAU..."
    secret_key = "7lafW..."
    session_token = "IQoJb3JpZ...EMb//..."

    ###########################################################################
    #
    # create connection
    #
    ###########################################################################

    provider = get_driver(Provider.EC2)
    conn = provider(access_id,
                    secret_key,
                    token=session_token,
                    region=region_name)

    ###########################################################################
    #
    # get image, flavor, network for instance creation
    #
    ###########################################################################

    print("Fetching images (AMI) list from AWS region. This will take a lot of seconds (AWS has a very long list of "
          "supported operating systems and versions)... please be patient...")
    images = conn.list_images()

    # image = ''
    # for img in images:
    #   # if img.name == ubuntu_image_name:
    #   if img.extra['owner_alias'] == 'amazon':
    #       print(img)
    #   if img.id == ubuntu_image_name:
    #       image = img

    # fetch/select the image referenced with ubuntu_image_name above
    image = [i for i in images if i.name == ubuntu_image_name][0]
    print(image)

    flavors = conn.list_sizes()
    flavor = [s for s in flavors if s.id == flavor_name][0]
    print(flavor)

    # networks = conn.ex_list_networks()
    # network = ''
    # for net in networks:
    #    if net.name == project_network:
    #        network = net

    ###########################################################################
    #
    # create keypair dependency
    #
    ###########################################################################

    print('Checking for existing SSH key pair...')
    keypair_exists = False
    for keypair in conn.list_key_pairs():
        if keypair.name == keypair_name:
            keypair_exists = True

    if keypair_exists:
        print('Keypair ' + keypair_name + ' already exists. Skipping import.')
    else:
        print('adding keypair...')
        conn.import_key_pair_from_file(keypair_name, pub_key_file)

    for keypair in conn.list_key_pairs():
        print(keypair)

    ###########################################################################
    #
    # create security group dependency
    #
    ###########################################################################

    print('Checking for existing worker security group...')
    worker_security_group_exists = False
    worker_security_group_name = 'worker'
    for security_group in conn.ex_get_security_groups():
        if security_group.name == worker_security_group_name:
            worker_security_group_id = security_group.id
            worker_security_group_exists = True

    if worker_security_group_exists:
        print('Worker Security Group ' + worker_security_group_name + ' already exists. Skipping creation.')
    else:
        worker_security_group_result = conn.ex_create_security_group('worker', 'for services that run on a worker node')
        worker_security_group_id = worker_security_group_result['group_id']
        conn.ex_authorize_security_group_ingress(worker_security_group_id, 22, 22, cidr_ips=['0.0.0.0/0'],
                                                 protocol='tcp')

    print('Checking for existing controller security group...')
    controller_security_group_exists = False
    controller_security_group_name = 'control'
    controller_security_group_id = ''
    for security_group in conn.ex_get_security_groups():
        if security_group.name == controller_security_group_name:
            controller_security_group_id = security_group.id
            controller_security_group_exists = True

    if controller_security_group_exists:
        print('Controller Security Group ' + controller_security_group_name + ' already exists. Skipping creation.')
    else:
        controller_security_group_result = conn.ex_create_security_group('control',
                                                                         'for services that run on a control node')
        controller_security_group_id = controller_security_group_result['group_id']
        conn.ex_authorize_security_group_ingress(controller_security_group_id, 22, 22, cidr_ips=['0.0.0.0/0'],
                                                 protocol='tcp')
        conn.ex_authorize_security_group_ingress(controller_security_group_id, 80, 80, cidr_ips=['0.0.0.0/0'],
                                                 protocol='tcp')
        conn.ex_authorize_security_group_ingress(controller_security_group_id, 5672, 5672,
                                                 group_pairs=[{'group_id': worker_security_group_id}], protocol='tcp')

    # for security_group in conn.ex_list_security_groups():
    #    print(security_group)

    ###########################################################################
    #
    # create app-controller
    #
    ###########################################################################

    # https://git.openstack.org/cgit/openstack/faafo/plain/contrib/install.sh
    # is currently broken, hence the "rabbitctl" lines were added in the example
    # below, see also https://bugs.launchpad.net/faafo/+bug/1679710
    #
    # Thanks to Stefan Friedmann for finding this fix ;)

    userdata = '''#!/usr/bin/env bash
    curl -L -s https://gogs.informatik.hs-fulda.de/srieger/cloud-computing-msc-ai-examples/raw/master/faafo/contrib/install.sh | bash -s -- \
        -i messaging -i faafo -r api
    rabbitmqctl add_user faafo guest
    rabbitmqctl set_user_tags faafo administrator
    rabbitmqctl set_permissions -p / faafo ".*" ".*" ".*"
    '''

    print('Starting new app-controller instance and wait until it is running...')
    instance_controller_1 = conn.create_node(name='app-controller',
                                             image=image,
                                             size=flavor,
                                             ex_keyname=keypair_name,
                                             ex_userdata=userdata,
                                             ex_security_groups=[controller_security_group_name])

    conn.wait_until_running(nodes=[instance_controller_1], timeout=120, ssh_interface='public_ips')

    ###########################################################################
    #
    # assign app-controller elastic ip
    #
    ###########################################################################

    # AWS offers elastic ips, that have the same function as floating IPs in OpenStack. However elastic IPs cost money,
    # and instances typically already have public IP in AWS, what a luxury ;)
    print('Checking for unused Elastic IP...')
    unused_elastic_ip = None
    for elastic_ip in conn.ex_describe_all_addresses():
        if not elastic_ip.instance_id:
            unused_elastic_ip = elastic_ip
            break

    if not unused_elastic_ip:
        print('Allocating new Elastic IP')
        unused_elastic_ip = conn.ex_allocate_address()
    conn.ex_associate_address_with_node(instance_controller_1, unused_elastic_ip)
    print('Controller Application will be deployed to http://%s' % unused_elastic_ip.ip)

    ###########################################################################
    #
    # getting id and ip address of app-controller instance
    #
    ###########################################################################

    # instance should not have a public ip? floating ips are assigned later
    # instance_controller_1 = conn.ex_get_node_details(instance_controller_1.id)
    # if instance_controller_1.public_ips:
    #    ip_controller = instance_controller_1.public_ips[0]
    # else:
    ip_controller = instance_controller_1.private_ips[0]

    ###########################################################################
    #
    # create app-worker-1
    #
    ###########################################################################

    userdata = '''#!/usr/bin/env bash
    curl -L -s https://gogs.informatik.hs-fulda.de/srieger/cloud-computing-msc-ai-examples/raw/master/faafo/contrib/install.sh | bash -s -- \
        -i faafo -r worker -e 'http://%(ip_controller)s' -m 'amqp://faafo:guest@%(ip_controller)s:5672/'
    ''' % {'ip_controller': ip_controller}

    print('Starting new app-worker-1 instance and wait until it is running...')
    instance_worker_1 = conn.create_node(name='app-worker-1',
                                         image=image,
                                         size=flavor,
                                         ex_keyname=keypair_name,
                                         ex_userdata=userdata,
                                         ex_security_groups=[worker_security_group_name])

    conn.wait_until_running(nodes=[instance_worker_1], timeout=120, ssh_interface='public_ips')

    ###########################################################################
    #
    # assign app-worker elastic ip
    #
    ###########################################################################

    # AWS offers elastic ips, that have the same function as floating IPs in OpenStack. However elastic IPs cost money,
    # and instances typically already have public IP in AWS, what a luxury ;)
    print('Checking for unused Elastic IP...')
    unused_elastic_ip = None
    for elastic_ip in conn.ex_describe_all_addresses():
        if not elastic_ip.instance_id:
            unused_elastic_ip = elastic_ip
            break

    if not unused_elastic_ip:
        print('Allocating new Elastic IP')
        unused_elastic_ip = conn.ex_allocate_address()
    conn.ex_associate_address_with_node(instance_worker_1, unused_elastic_ip)
    print('The worker will be available for SSH at %s' % unused_elastic_ip.ip)

    print('You can use ssh to login to the controller using your private key. After login, you can list available '
          'fractals using "faafo list". To request the generation of new fractals, you can use "faafo create". '
          'You can also see other options to use the faafo example cloud service using "faafo -h".')


if __name__ == '__main__':
    main()
